unit MailRcv1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Pop3Prot, WSocket, MimeDec, ExtCtrls;

const
   CrLf = #13#10;

type
  TForm1 = class(TForm)
    Pop3Cli: TPop3Cli;
    UserN: TEdit;
    PassW: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    HostName: TEdit;
    Label3: TLabel;
    GetMailBtn: TButton;
    Memo1: TMemo;
    MimeDecode: TMimeDecode;
    AbortBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure GetMailBtnClick(Sender: TObject);
    procedure Pop3CliDisplay(Sender: TObject; Msg: String);
    procedure Pop3CliRequestDone(Sender: TObject; RqType: TPop3Request; Error: Word);
    procedure Pop3CliMessageLine(Sender: TObject);
    procedure Pop3CliMessageBegin(Sender: TObject);
    procedure Pop3CliMessageEnd(Sender: TObject);
    procedure Pop3CliSessionClosed(Sender: TObject; Error: Word);
    procedure MimeDecodePartLine(Sender: TObject; Data: PChar; DataLen: Integer);
    procedure MimeDecodeHeaderEnd(Sender: TObject);
    procedure MimeDecodeMessageEnd(Sender: TObject);
    procedure AbortBtnClick(Sender: TObject);
  private
    FLineBuf: array [0..255] of char;
    FCharCnt: Integer;
    FMsg: TMemoryStream;
    procedure Log(const ToLog: string);
  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
   FMsg := TMemoryStream.Create;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
   FMsg.Free;
end;

procedure TForm1.Log(const ToLog: string);
begin
   Memo1.Lines.Add(ToLog);
end;

procedure TForm1.AbortBtnClick(Sender: TObject);
begin
   Pop3Cli.Abort;
end;

procedure TForm1.Pop3CliDisplay(Sender: TObject; Msg: String);
begin
   Log(Msg);
end;

procedure TForm1.GetMailBtnClick(Sender: TObject);
begin
   with Pop3Cli do begin
      Host     := HostName.Text;
      UserName := UserN.Text;
      PassWord := PassW.Text;
      Open;
   end;
end;

procedure TForm1.Pop3CliRequestDone(Sender: TObject; RqType: TPop3Request; Error: Word);
begin
   with Sender as TPop3Cli do begin
      if Error <> 0 then begin
         if Error >= 10000 then
            Log(Format('WinSock error %d %s', [Error, WSocketErrorDesc(Error)]))
         else
            Log(Format('POP3 error %d', [Error]));
         Abort;
         Exit;
      end;

      case RqType of
         Pop3Open: Stat;

         Pop3Stat:
            if MsgCount < 1 then Quit
            else begin
               MsgNum := 1;
               Retr;
            end;

         Pop3Retr: Dele;

         Pop3Dele:
            if MsgNum = MsgCount then Quit
            else begin
               MsgNum := MsgNum + 1;
               Retr;
            end;

         else Abort;

      end; // case
   end;
end;

procedure TForm1.Pop3CliMessageBegin(Sender: TObject);
begin
   FMsg.Clear;
end;

procedure TForm1.Pop3CliMessageLine(Sender: TObject);
begin
   with Sender as TPop3Cli do
      FMsg.Write(LastResponse[1], Length(LastResponse));
   FMsg.Write(CrLf[1], 2);
end;

procedure TForm1.Pop3CliMessageEnd(Sender: TObject);
begin
   FMsg.Seek(0, soFromBeginning);
   MimeDecode.DecodeStream(FMsg);
end;

procedure TForm1.Pop3CliSessionClosed(Sender: TObject; Error: Word);
begin
   if Error <> 0 then
      Log(Format('WinSock error %d %s', [Error, WSocketErrorDesc(Error)]));
   Log('Disconnected');
end;

procedure TForm1.MimeDecodeHeaderEnd(Sender: TObject);
begin
   with Sender as TMimeDecode do begin
      Log('---- header -----');
      Log('From    ' + From);
      Log('To      ' + Dest);
      Log('Date    ' + Date);
      Log('Reply   ' + ReturnPath);
      Log('Subject ' + Subject);
      Log('Content ' + ContentType);
      Log('---- message ----');
   end;
end;

procedure TForm1.MimeDecodePartLine(Sender: TObject; Data: PChar; DataLen: Integer);
var i: integer;
begin
   i := 0;
   while i < DataLen do begin
      if Data[i] = #13 then
         Inc(i)
      else if Data[i] = #10 then begin
         FLineBuf[FCharCnt] := #0;
         Log(StrPas(FLineBuf));
         FCharCnt := 0;
         Inc(i);
      end
      else begin
         FLineBuf[FCharCnt] := Data[i];
         Inc(FCharCnt);
         Inc(i);
      end;
      if FCharCnt >= (High(FLineBuf) - 1) then begin
         FLineBuf[High(FLineBuf) - 1] := #0;
         Log(StrPas(FLineBuf));
         FCharCnt := 0;
      end;
   end;
end;

procedure TForm1.MimeDecodeMessageEnd(Sender: TObject);
begin
   Log('-----  end  -----');
end;

end.
